// -*- C++ -*-
/*!
 *
 * @file BufferBase.h
 * @brief Buffer abstract class
 * @date $Date: 2007-12-31 03:06:12 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: BufferBase.h 706 2008-04-16 00:36:15Z n-ando $
 */

#ifndef BufferBase_h
#define BufferBase_h

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class BufferBase
   * @brief BufferBase ۃNX
   * 
   * X̃obt@̂߂̒ۃC^[tF[XNXB
   * ۃobt@NX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   * \<DataType\>Ƃăobt@ŕێf[^^w肷B
   *
   * publicC^[tF[XƂĈȉ̂̂񋟂B
   * - write(): obt@ɏ
   * - read(): obt@ǂݏo
   * - length(): obt@Ԃ
   * - isFull(): obt@tł
   * - isEmpty(): obt@ł
   *
   * protectedC^[tF[XƂĈȉ̂̂񋟂B
   * - put(): obt@Ƀf[^
   * - get(): obt@f[^ǂݏo
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class BufferBase
   * @brief BufferBase abstract class
   *
   * This is the abstract interface class for various Buffer.
   * Concrete buffer classes must implement the following pure virtual
   * functions.
   * The users specify data type to hold it in a buffer as \<DataType\>.
   *
   * This class provides public interface as follows.
   * - write(): Write data into the buffer.
   * - read(): Read data from the buffer.
   * - length(): Get the buffer length.
   * - isFull(): Check on whether the buffer is full.
   * - isEmpty(): Check on whether the buffer is empty.
   *
   * This class provides protected interface as follows.
   * - put(): Store data into the buffer.
   * - get(): Get data from the buffer.
   *
   * @param DataType Data type to be stored to the buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class BufferBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     *
     * @brief Virtual destructor
     *
     * @endif
     */
    virtual ~BufferBase()
    {
    };
    
    /*!
     * @if jp
     *
     * @brief obt@̒擾
     * 
     * obt@擾邽߂̏z֐
     * 
     * @return obt@
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * Pure virtual function to get the buffer length.
     *
     * @return buffer length
     * 
     * @endif
     */
    virtual long int length() const = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * obt@Ƀf[^ނ߂̏z֐
     * 
     * @param value ݑΏۃf[^
     *
     * @return f[^݌(true:ݐCfalse:ݎs)
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Pure virtual function to write data into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool write(const DataType& value) = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@f[^ǂݏo߂̏z֐
     * 
     * @param value ǂݏof[^
     *
     * @return f[^ǂݏo(true:ǂݏoCfalse:ǂݏos)
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Pure virtual function to read data from the buffer.
     *
     * @param value Read data.
     *
     * @return Result of having read (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool read(DataType& value) = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@full`FbN
     * 
     * obt@full`FbNpz֐
     *
     * @return full`FbN(true:obt@fullCfalse:obt@󂫂)
     * 
     * @else
     *
     * @brief Check on whether the buffer is full.
     *
     * Pure virtual function to check on whether the buffer is full.
     *
     * @return True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool isFull() const = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@empty`FbN
     * 
     * obt@empty`FbNpz֐
     *
     * @return empty`FbN(true:obt@emptyCfalse:obt@f[^)
     * 
     * @else
     *
     * @brief Check on whether the buffer is empty.
     *
     * Pure virtual function to check on whether the buffer is empty.
     *
     * @return True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool isEmpty() const = 0;
    
  protected:
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^i[
     * 
     * obt@ւ̃f[^i[pz֐
     * 
     * @param data Ώۃf[^
     * 
     * @else
     *
     * @brief Store data into the buffer
     *
     * Pure virtual function to store data into the buffer.
     *
     * @param data Target data
     * 
     * @endif
     */
    virtual void put(const DataType& data) = 0;
    
    /*!
     * @if jp
     *
     * @brief obt@f[^擾
     * 
     * obt@Ɋi[ꂽf[^擾pz֐
     *
     * @return 擾f[^
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * Pure virtual function to get data from the buffer.
     *
     * @return Data got from buffer.
     *
     * @endif
     */
    virtual const DataType& get() = 0;
    
    /*!
     * @if jp
     *
     * @brief ɏރobt@ւ̎QƂ擾
     * 
     * ݃obt@ւ̎QƎ擾pz֐
     *
     * @return ̏ݑΏۃobt@ւ̎Q
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * Pure virtual function to get the buffer's reference to be written 
     * the next.
     *
     * @return Reference to be written the next
     * 
     * @endif
     */
    virtual DataType& getRef() = 0;
  };
  
  /*!
   * @if jp
   * @class NullBuffer
   * @brief _~[obt@NX
   * 
   * obt@PŒ̃_~[obt@NXB
   * \<DataType\>Ƃăobt@ŕێf[^^w肷B
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class NullBuffer
   * @brief Concrete buffer class for dummy
   * 
   * Concrete buffer class for dummy. Buffer length is fixed to 1.
   * The users specify data type to hold it in a buffer as \<DataType\>.
   *
   * @param DataType Data type to hold in a buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class NullBuffer
    : public BufferBase<DataType>
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     * obt@1(Œ)ŏB
     *
     * @param size obt@()
     * 
     * @else
     *
     * @brief Constructer
     * 
     * Constructer.
     * Initialize buffer length to always 1.
     *
     * @param size Buffer length(Not use)
     * 
     * @endif
     */
    NullBuffer(long int size = 1)
      : m_length(1)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     * 
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destractor
     *
     * @endif
     */
    virtual ~NullBuffer()
    {
    }
    
    /*!
     * @if jp
     *
     * @brief obt@(1Œ)擾
     * 
     * obt@擾B(1ԂB)
     * 
     * @return obt@(1Œ)
     * 
     * @else
     *
     * @brief Get the buffer length (always 1)
     *
     * Get the buffer length. (Return always 1.)
     *
     * @return buffer length(always 1)
     * 
     * @endif
     */
    virtual long int length() const
    {
      return 1;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * ŗ^ꂽf[^obt@ɏށB
     * 
     * @param value ݑΏۃf[^
     *
     * @return f[^݌(true:ݐCfalse:ݎs)
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * Write data which were given with an argument into the buffer.
     *
     * @param value Target data to write.
     *
     * @return Result of having written in data (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool write(const DataType& value)
    {
      m_data = value;
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@f[^ǂݏo
     * 
     * obt@Ɋi[ꂽf[^ǂݏoB
     * 
     * @param value ǂݏof[^
     *
     * @return f[^ǂݏo(true:ǂݏoCfalse:ǂݏos)
     * 
     * @else
     *
     * @brief Read data from the buffer
     *
     * Read data stored in the buffer.
     *
     * @param value Read data.
     *
     * @return Result of having read (true:Successful, false:Failed)
     *
     * @endif
     */
    virtual bool read(DataType& value)
    {
      value = m_data;
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@full`FbN
     * 
     * obt@full`FbNB(falseԂB)
     *
     * @return full`FbN(false)
     * 
     * @else
     *
     * @brief Check on whether the buffer is full.
     *
     * Check on whether the buffer is full. (Always false.)
     *
     * @return Always false.
     *
     * @endif
     */
    virtual bool isFull() const
    {
      return false;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@empty`FbN
     * 
     * obt@empty`FbNB(falseԂB)
     *
     * @return empty`FbN(false)
     * 
     * @else
     *
     * @brief Check on whether the buffer is empty.
     *
     * Check on whether the buffer is empty. (Always false.)
     *
     * @return Always false.
     *
     * @endif
     */
    virtual bool isEmpty() const
    {
      return false;
    }
    
  protected:
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^i[
     * 
     * ŗ^ꂽf[^obt@Ɋi[B
     * 
     * @param data Ώۃf[^
     * 
     * @else
     *
     * @brief Store data into the buffer
     *
     * Store data which were given with an argument into the buffer.
     *
     * @param data Target data to store.
     *
     * @endif
     */
    virtual void put(const DataType& data)
    {
      m_data = data;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@f[^擾
     * 
     * obt@Ɋi[ꂽf[^擾B
     *
     * @return 擾f[^
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * Get data from the buffer.
     *
     * @return Data got from buffer.
     *
     * @endif
     */
    virtual const DataType& get()
    {
      return m_data;
    }
    
    /*!
     * @if jp
     *
     * @brief ɏރobt@ւ̎QƂ擾
     * 
     * ݃obt@ւ̎QƂ擾B
     * {obt@ł̓obt@͌ŒłPł邽߁C
     * ɓʒuւ̎QƂԂB
     *
     * @return ̏ݑΏۃobt@ւ̎Q(Œ)
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * Get the reference to be written buffer.
     * Return always same position because this buffer's length is always 1.
     *
     * @return Reference to be written the next(always same)
     *
     * @endif
     */
    virtual DataType& getRef()
    {
      return m_data;
    }
    
  private:
    DataType m_data;
    long int m_length;
  };
}; // namespace RTC
#endif // BufferBase_h
