// -*- C++ -*-
/*!
 * @file  InPortCorbaConsumer.h
 * @brief InPortCorbaConsumer class
 * @date  $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortCorbaConsumer.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef InPortCorbaConsumer_h
#define InPortCorbaConsumer_h

#include <rtm/idl/DataPortSkel.h>
#include <rtm/BufferBase.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/InPortConsumer.h>
#include <iostream>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPortCorbaConsumer
   *
   * @brief InPortCorbaConsumer NX
   *
   * ʐMi CORBA 𗘗p̓|[gRV[}̎NXB
   *
   * @param DataType {|[gɂĈf[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortCorbaConsumer
   *
   * @brief InPortCorbaConsumer class
   *
   * This is an implementation class of the input port Consumer 
   * that uses CORBA for means of communication.
   *
   * @param DataType Data type for this port
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class InPortCorbaConsumer
    : public InPortConsumer,
      public CorbaConsumer<RTC::InPortAny>
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param buffer YRV[}Ɋ蓖Ăobt@IuWFNg
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param buffer The buffer object that is attached to this Consumer
     *
     * @endif
     */
    InPortCorbaConsumer(BufferBase<DataType>& buffer)
      : m_buffer(buffer)
    {
    }
    
    /*!
     * @if jp
     * @brief Rs[RXgN^
     *
     * Rs[RXgN^
     *
     * @param consumer Rs[ InPortCorbaConsumer IuWFNg
     *
     * @else
     * @brief Copy constructor
     *
     * Copy constructor
     *
     * @param consumer InPortCorbaConsumer object of copy source
     *
     * @endif
     */
    InPortCorbaConsumer(const InPortCorbaConsumer<DataType>& consumer)
      : CorbaConsumer<RTC::InPortAny>(consumer), m_buffer(consumer.m_buffer)
    {
    }
    
    /*!
     * @if jp
     * @brief Zq
     *
     * Zq
     *
     * @param consumer  InPortCorbaConsumer IuWFNg
     *
     * @return 
     *
     * @else
     * @brief Assignment operator
     *
     * Assignment operator
     *
     * @param consumer InPortCorbaConsumer object of assignment source
     *
     * @return The assignment result
     *
     * @endif
     */
    InPortCorbaConsumer&
    operator=(const InPortCorbaConsumer<DataType>& consumer)
    {
      if (this == &consumer) return *this;
      m_buffer = consumer.m_buffer;
    }
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortCorbaConsumer()
    {}
    
    /*!
     * @if jp
     * @brief obt@ւ̃f[^
     *
     * obt@Ƀf[^
     *
     * @param data Ώۃf[^
     *
     * @else
     * @brief Write data into the buffer
     *
     * Write data into the buffer.
     *
     * @param data The target data for writing
     *
     * @endif
     */
    void put(DataType& data)
    {
      CORBA::Any tmp;
      tmp <<= data;
      _ptr()->put(tmp);
    }
    
    /*!
     * @if jp
     * @brief obt@̃f[^o
     *
     * obt@f[^ođoB
     *
     * @else
     * @brief Read data from the buffer
     *
     * Read data from the buffer and send it.
     *
     * @endif
     */
    void push()
    {
      DataType data;
      CORBA::Any tmp;
      m_buffer.read(data);
      tmp <<= data;
      
      // {̓G[ׂ
      if (CORBA::is_nil(_ptr())) return;
      try
	{
	  _ptr()->put(tmp);
	}
      catch(...)
	{
	  //hoge IuWFNgɂȂdisconnectׂ
	  return;
	}
    }
    
    /*!
     * @if jp
     * @brief Rs[̐
     *
     * YInPortCorbaConsumerIuWFNg̕𐶐B
     *
     * @return Rs[ꂽInPortCorbaConsumerIuWFNg
     *
     * @else
     * @brief Generate clone
     *
     * Clone this InPortCorbaConsumer object.
     *
     * @return The clone InPortCorbaConsumer object
     *
     * @endif
     */
    virtual InPortCorbaConsumer* clone() const
    {
      return new InPortCorbaConsumer<DataType>(*this);
    }
    
    /*!
     * @if jp
     * @brief f[^Mʒmւ̓o^
     *
     * w肳ꂽvpeBɊÂāAf[^oʒm̎󂯎ɓo^B
     *
     * @param properties o^
     *
     * @return o^(o^:trueAo^s:false)
     *
     * @else
     * @brief Subscribe to the data sending notification
     *
     * Subscribe to the data sending notification based on specified 
     * property information.
     *
     * @param properties Information for subscription
     *
     * @return Subscription result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties)
    {
      if (!NVUtil::isStringValue(properties,
				 "dataport.dataflow_type",
				 "Push"))
	{
	  return false;
	}
      
      CORBA::Long index;
      index = NVUtil::find_index(properties,
				 "dataport.corba_any.inport_ref");
      if (index < 0)
	{
	  return false;
	}
      
      CORBA::Object_ptr obj;
      if (properties[index].value >>= CORBA::Any::to_object(obj))
	{
	  if (!CORBA::is_nil(obj))
	    {
	      setObject(obj);
	      return true;
	    }
	}
      return false;;
    }
    
    /*!
     * @if jp
     * @brief f[^Mʒm̓o^
     *
     * f[^oʒm̎󂯎肩o^B
     *
     * @param properties o^
     *
     * @else
     * @brief Unsubscribe the data send notification
     *
     * Unsubscribe the data send notification.
     *
     * @param properties Information for unsubscription
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties)
    {
      ;
    }
    
  private:
    BufferBase<DataType>& m_buffer;
  };
};     // namespace RTC
#endif // InPortCorbaConsumer_h
