// -*- C++ -*-
/*!
 * @file  InPortTcpSockProvider.h
 * @brief InPortTcpSockProvider class
 * @date  $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortTcpSockProvider.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef InPortTcpSockProvider_h
#define InPortTcpSockProvider_h

#include <rtm/BufferBase.h>
#include <rtm/InPortProvider.h>
#include <rtm/TcpSockServer.h>
#include <rtm/StringUtil.h>

namespace RTC
{
  /*!
   * @if jp
   * @class InPortTcpSockProvider
   * @brief InPortTcpSockProvider NX
   *
   * ʐMi TCP \Pbg𗘗p̓|[gvoC_̎NXB
   *
   * @param DataType YvoC_Ɋ蓖Ăobt@ێf[^^
   *
   * @since 0.4.1
   *
   * @else
   * @class InPortTcpSockProvider
   * @brief InPortTcpSockProvider class
   *
   * This is an implementation class of OutPort Provider 
   * that uses TCP socket for means of communication.
   *
   * @param DataType Data type held by the buffer that attached to this provider
   *
   * @since 0.4.1
   *
   * @endif
   */
  template <class DataType>
  class InPortTcpSockProvider
    : public InPortProvider
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     * |[gvpeBɈȉ̍ڂݒ肷B
     *  - C^[tF[X^Cv : TCP_Any
     *  - f[^t[^Cv : Push
     *  - TuXNvV^Cv : Any
     *
     * @param buffer YvoC_Ɋ蓖Ăobt@IuWFNg
     * @param prop TCP ʐMݒpvpeB
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     * Set the following items to port properties
     *  - Interface type : TCP_Any
     *  - Data flow type : Push
     *  - Subscription type : Any
     *
     * @param buffer Buffer object that is attached to this provider
     * @param prop Property for setup TCP communication.
     *
     * @endif
     */
    InPortTcpSockProvider(BufferBase<DataType>& buffer, Properties& prop)
      : m_server(buffer, prop), m_buffer(buffer), m_prop(prop)
    {
      CORBA::Any any_var;
      DataType   tmp_var;
      any_var <<= tmp_var;
      
      // PortProfile setting
      setDataType(any_var.type()->name());
      setInterfaceType("TCP_Any");
      setDataFlowType("Push");
      setSubscriptionType("Any");
      
      // setup TCP server
      m_server.open(0);
      
      // set the TCP server address to properties
      std::string addr;
      addr  = m_server.getHostName();
      addr += ":";
      addr += otos(m_server.getPortNumber());
      CORBA_SeqUtil::push_back(m_properties,
			       NVUtil::newNV("dataport.tcp_any.inport_addr", addr.c_str()));
    }
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortTcpSockProvider()
    {
    }
    
    /*!
     * @if jp
     * @brief InterfaceJ
     *
     * InterfaceJB
     *
     * @param prop Interface󂯎vpeB
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information
     *
     * @param prop Property for receiving interface information
     *
     * @endif
     */
    void publishInterface(SDOPackage::NVList& prop)
    {
      if (!NVUtil::isStringValue(prop,
				 "dataport.interface_type",
				 "TCP_Any"))
	{
	  return;
	}
      SDOPackage::NVList nv(m_properties);
      NVUtil::append(prop, nv);
    }
    
  private:
    TcpServer<DataType> m_server;
    BufferBase<DataType>& m_buffer;
    Properties& m_prop;
  };
};     // namespace RTC
#endif // InPortTcpSockProvider_h
