// -*- C++ -*-
/*!
 * @file ObjectManager.h
 * @brief Object management class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2007
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ObjectManager.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef ObjectManager_h
#define ObjectManager_h

#include <rtm/RTC.h>

#include <vector>
#include <string>
#include <algorithm>

#include <ace/Thread.h>
#include <ace/Synch.h>

/*!
 * @if jp
 *
 * @brief IuWFNgǗpNX
 *
 * eIuWFNgǗ邽߂̃NXB
 *
 * @since 0.4.0
 *
 * @else
 *
 * @brief Class for managing objects
 *
 * This is a class for managing various objects.
 *
 * @since 0.4.0
 *
 * @endif
 */
template <typename Identifier, typename Object, typename Predicate>
class ObjectManager
{
public:
  typedef std::vector<Object*>                  ObjectVector;
  typedef typename ObjectVector::iterator       ObjectVectorItr;
  typedef typename ObjectVector::const_iterator ObjectVectorConstItr;
  
  /*!
   * @if jp
   *
   * @brief RXgN^
   * 
   * RXgN^
   * 
   * @else
   *
   * @brief Constructor
   * 
   * Constructor
   * 
   * @endif
   */
  ObjectManager(){};
  
  /*!
   * @if jp
   *
   * @brief fXgN^
   * 
   * fXgN^
   * 
   * @else
   *
   * @brief Destructor
   * 
   * Destructor
   * 
   * @endif
   */
  ~ObjectManager(){};
  
  /*!
   * @if jp
   *
   * @brief w肵IuWFNgo^
   * 
   * w肵IuWFNgo^B
   * IuWFNgo^ς݂̏ꍇ́AsȂB
   *
   * @param obj o^ΏۃIuWFNg
   *
   * @return o^(IuWFNgo^ꍇtrue)
   * 
   * @else
   *
   * @brief Register the specified object
   * 
   * Register the object that was specified.
   * If the same object is already registered, this does not anything.
   *
   * @param obj The target object for the registration
   *
   * @return Registration result (True if object was registerd)
   * 
   * @endif
   */
  bool registerObject(Object* obj)
  {
    ObjectVectorItr it;
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    
    it = std::find_if(m_objects._obj.begin(), m_objects._obj.end(),
		      Predicate(obj));
    if (it == m_objects._obj.end())
      {
	m_objects._obj.push_back(obj);
	return true;
      }
    return false;
  }
  
  /*!
   * @if jp
   *
   * @brief w肵IuWFNgo^
   * 
   * w肵IuWFNg̓o^A擾B
   * w肵IuWFNgo^ĂȂꍇɂNULLԂB
   *
   * @param id o^ΏۃIuWFNgID
   *
   * @return o^ꂽIuWFNg
   * 
   * @else
   *
   * @brief Unregister the specified object
   * 
   * Unregister the object that was specified and get it.
   * This operation returns NULL if the specified object is not 
   * registered.
   *
   * @param id ID of the target object for the unregistration
   *
   * @return Unregistered object
   * @endif
   */
  Object* unregisterObject(const Identifier& id)
  {
    ObjectVectorItr it;
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    
    it = std::find_if(m_objects._obj.begin(), m_objects._obj.end(),
		      Predicate(id));
    if (it != m_objects._obj.end())
      {
	Object* obj(*it);
	m_objects._obj.erase(it);
	return obj;
      }
    return NULL;;
  }
  
  /*!
   * @if jp
   *
   * @brief IuWFNg
   * 
   * o^ĂIuWFNg̒w肵ɍvIuWFNg
   * Ď擾B
   * w肵ɍvIuWFNgo^ĂȂꍇɂNULLԂB
   *
   * @param id ΏۃIuWFNgID
   *
   * @return IuWFNǧ
   * 
   * @else
   *
   * @brief Find the object
   * 
   * Find the object that matches the specified condition among the registered
   * objects and get it.
   * This operation returns NULL if the object that does matches the specified
   * condition is not registered.
   *
   * @param id ID of the target object for finding
   *
   * @return Result of finding object
   * 
   * @endif
   */
  Object* find(const Identifier& id) const
  {
    ObjectVectorConstItr it;
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    it = std::find_if(m_objects._obj.begin(), m_objects._obj.end(),
		      Predicate(id));
    if (it != m_objects._obj.end())
      {
	return *it;
      }
    return NULL;
  }
  
  /*!
   * @if jp
   *
   * @brief o^ĂIuWFNg̃Xg擾
   * 
   * o^ĂIuWFNg̃Xg擾B
   *
   * @return o^ĂIuWFNgEXg
   * 
   * @else
   *
   * @brief Get a list of obejects that are registerd
   * 
   * Get a list of objects that are registerd.
   *
   * @return List of registerd objects.
   * 
   * @endif
   */
  std::vector<Object*> getObjects() const
  {
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    return m_objects._obj;
  }
  
  /*!
   * @if jp
   * @brief IuWFNgpt@N^
   * @else
   * @brief Functor for searching object
   * @endif
   */
  template <class Pred>
  Pred for_each(Pred p)
  {
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    return std::for_each(m_objects._obj.begin(), m_objects._obj.end(), p);
  }
  
  /*!
   * @if jp
   * @brief IuWFNgpt@N^
   * @else
   * @brief Functor for searching object
   * @endif
   */
  template <class Pred>
  Pred for_each(Pred p) const
  {
    ACE_Guard<ACE_Thread_Mutex> guard(m_objects._mutex);
    return std::for_each(m_objects._obj.begin(), m_objects._obj.end(), p);
  }
  
protected:
  /*!
   * @if jp
   * @brief IuWFNgǗp\
   * @else
   * @brief The structure for object management
   * @endif
   */
  struct Objects
  {
    mutable ACE_Thread_Mutex _mutex;
    ObjectVector _obj;
  };
  /*!
   * @if jp
   * @brief o^ς݃IuWFNgEXg
   * @else
   * @brief The list of registered objects
   * @endif
   */
  Objects m_objects;
};
#endif // ObjectManager_h
