// -*- C++ -*-
/*!
 * @file PortCallBack.h
 * @brief PortCallBack class
 * @date $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortCallBack.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef PortCallBack_h
#define PortCallBack_h

namespace RTC
{
  //============================================================
  // callback functor base classes
  /*!
   * @if jp
   * @class OnWrite
   * @brief write() ̃R[obNۃNX
   *
   * DataPort̃obt@Ƀf[^write()钼OɌĂяoR[obNp
   * C^[tF[XB
   *
   * @param DataType obt@ɏރf[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWrite
   * @brief Callback abstract class on write()
   *
   * This is the interface for callback invoked immediately before
   * data is done write() into the DataPort's buffer.
   *
   * @param DataType Data type to write into the buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWrite
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWrite(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@Ƀf[^܂钼OɌĂяoR[obN\bh
     *
     * @param value obt@ɏ܂f[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked immediately before data is written
     * into the buffer.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnWriteConvert
   * @brief write() ̃f[^ϊR[obNۃNX
   *
   * InPort/OutPort̃obt@Ƀf[^ write()鎞ɌĂяo
   * R[obNpC^[tF[XB
   * ̃R[obN̖߂lobt@Ɋi[B
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWriteConvert
   * @brief Data convert callback abstract class on write()
   *
   * This is the interface for callback invoked when data is done write()
   * into the InPort/OutPort's buffer.
   * The return value of this callback will be stored in the buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWriteConvert
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWriteConvert(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@Ƀf[^܂ۂɌĂяoR[obN\bh
     *
     * @param value ϊOf[^
     *
     * @return ϊf[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when data is written into the
     * buffer.
     *
     * @param value Data before the convert
     *
     * @return Data after the convert
     *
     * @endif
     */
    virtual DataType operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnRead
   * @brief read() ̃R[obNۃNX
   *
   * InPort/OutPort̃obt@f[^ read()钼ɌĂяo
   * R[obNpC^[tF[XB
   *
   * @since 0.4.0
   *
   * @else
   * @class OnRead
   * @brief Callback abstract class on read()
   *
   * This is the interface for callback invoked immediately before
   * data is done read() from the InPort/OutPort's buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnRead
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnRead(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@f[^ǂݏo钼OɌĂяoR[obN\bh
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked immediately before data is readout
     * from the buffer.
     *
     * @endif
     */
    virtual void operator()() = 0;
  };
  
  /*!
   * @if jp
   * @class OnReadConvert
   * @brief read() ̃f[^ϊR[obNۃNX
   *
   * InPort/OutPort̃obt@f[^ read()ۂɌĂяo
   * R[obNpC^[tF[XB
   * ̃R[obN̖߂lread()̖߂lƂȂB
   *
   * @since 0.4.0
   *
   * @else
   * @class OnReadConvert
   * @brief Data convert callback abstract class on read()
   *
   * This is the interface for callback invoked when data is done read()
   * from the InPort/OutPort's buffer.
   * The return value of this callback will be the return value of read().
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnReadConvert
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnReadConvert(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@f[^ǂݏoۂɌĂяoR[obN\bh
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when data is readout from the
     * buffer.
     *
     * @endif
     */
    virtual DataType operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnOverflow
   * @brief obt@I[o[t[̃R[obNۃNX
   *
   * InPort/OutPort̃obt@Ƀf[^put()鎞Aobt@I[o[t[
   * ꍇɌĂ΂R[obN\bhB
   *
   * @since 0.4.0
   *
   * @else
   * @class OnOverflow
   * @brief Callback abstract class when the buffer overflow occurs
   *
   * This is the callback method invoked when data is done put()
   * into the InPort/OutPort's buffer or when the buffer overflow occurs.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnOverflow
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnOverflow(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@I[o[t[ɌĂяoR[obN\bh
     *
     * @param value obt@֏ރf[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the buffer overflow occurs.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnUnderflow
   * @brief Underflow ̃R[obNۃNX
   *
   * @since 0.4.0
   *
   * InPort/OutPort̃obt@Ƀf[^read()鎞ɁAǂݏoׂf[^
   * ȂꍇɌĂяoR[obNC^tF[XB
   * ̃R[obN̖߂lread()̖߂lƂȂB
   *
   * @else
   * @class OnUnderflow
   * @brief Callback abstract class on underflow
   *
   * @since 0.4.0
   *
   * This is the interface for callback invoked when data is done read()
   * into the InPort/OutPort's buffer and the readout data does not exist.
   * The return value of this callback will be the return value of read().
   *
   * @endif
   */
  template <class DataType>
  struct OnUnderflow
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnUnderflow(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * obt@A_[t[ɌĂяoR[obN\bh
     *
     * @return ւƂȂǂݏof[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the buffer underflow occurs.
     *
     * @return Substituted readout data
     *
     * @endif
     */
    virtual DataType operator()() = 0;
  };
  
  /*!
   * @if jp
   * @class OnWriteTimeout
   * @brief ^CAEg̃R[obNۃNX
   *
   * InPort/OutPort̃obt@Ƀf[^write()ۂɁA^CAEg
   * ɌĂяoR[obNC^tF[XB
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWriteTimeout
   * @brief Callback abstract class on timeout
   *
   * This is the interface for callback invoked when data is done write()
   * into the InPort/OutPort's buffer and the timeout occurred.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWriteTimeout
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWriteTimeout(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * ^CAEgɌĂяoR[obN\bh
     *
     * @param value obt@֏ރf[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the timeout occurs.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnReadTimeout
   * @brief ^CAEg̃R[obNۃNX
   *
   * InPort/OutPort̃obt@Ƀf[^read()ۂɁA^CAEg
   * ɌĂяoR[obNC^tF[XB
   *
   * @since 0.4.0
   *
   * @else
   * @class OnReadTimeout
   * @brief OnReadTimeout abstract class
   *
   * This is the interface for callback invoked when data is done read()
   * into the InPort/OutPort's buffer and the timeout occurred.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnReadTimeout
  {
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnReadTimeout(){}
    
    /*!
     * @if jp
     *
     * @brief R[obN\bh
     *
     * ^CAEgɌĂяoR[obN\bh
     *
     * @return ւƂȂǂݏof[^
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the timeout occurs.
     *
     * @return Substituted readout data
     *
     * @endif
     */
    virtual DataType operator()() = 0;
  };
};
#endif // PortCallBack_h
