// -*- C++ -*-
/*!
 * @file  PublisherPeriodic.h
 * @brief PublisherPeriodic class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherPeriodic.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef PublisherPeriodic_h
#define PublisherPeriodic_h

#include <rtm/RTC.h>

#include <ace/Task.h>
#include <ace/Synch.h>
#include <rtm/PublisherBase.h>

namespace RTC
{
  class InPortConsumer;
  class Properties;
  /*!
   * @if jp
   * @class PublisherPeriodic
   * @brief PublisherPeriodic NX
   *
   * ŃRV[}̑oĂяo Publisher
   * IɃf[^MsꍇɎgpB
   *
   * @else
   * @class PublisherPeriodic
   * @brief PublisherPeriodic class
   *
   * Publisher that invokes the consumer's sending process periodically.
   * This is used when the data sending is regularly executed.
   *
   * @endif
   */
  class PublisherPeriodic
    : public PublisherBase,
      public ACE_Task<ACE_MT_SYNCH>
  {
    //    class InPortConsumer;
    
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     * ǒĂяoԊuAPropertyIuWFNgdataport.push_rateo
     * ɐݒ肵ĂKvBoԊúAHzPʂ̕ŎwB
     * Ƃ΁A1000.0Hz̏ꍇ́Au1000.0vݒB
     * LvpeBݒ̏ꍇ́Au1000HzvݒB
     *
     * @param consumer f[^o҂RV[}
     * @param property {Publisher̋쓮ݒ肵PropertyIuWFNg
     *
     * @else
     * @brief Constructor
     *
     * Constructor.
     * The intervals of invoking send processing needs to be set in
     * dataport.push_rate of Property object. The interval is specified by
     * floating point string in Hz.
     * For example, "1000.0" is set for 1000.0Hz.
     * If the above property is unset, "1000Hz" will be set.
     *
     * @param consumer Consumer which waits for data sending
     * @param property Property objects that are set the control information
     *                 of this Publisher
     *
     * @endif
     */
    PublisherPeriodic(InPortConsumer* consumer,
		      const Properties& property);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PublisherPeriodic();
    
    /*!
     * @if jp
     * @brief Observer֐
     *
     * { Publisher ł͉sȂB
     *
     * @else
     * @brief Observer function
     *
     * Execute nothing in this Publisher.
     *
     * @endif
     */
    virtual void update();
    
    /*!
     * @if jp
     * @brief Xbhs֐
     *
     * ACE_Task::svc() ̃I[o[Ch
     *
     * @else
     * @brief Thread execution function
     *
     * ACE_Task::svc() override function.
     *
     * @endif
     */
    virtual int svc(void);
    
    /*!
     * @if jp
     * @brief ^XNJn
     *
     * ACE_Task::open() ̃I[o[Ch
     *
     * @else
     * @brief Start task
     *
     * ACE_Task::open() override function.
     *
     * @endif
     */
    virtual int open(void *args);
    
    /*!
     * @if jp
     * @brief ^XNI֐
     *
     * ACE_Task::release() ̃I[o[Ch
     * 쓮tOfalseɐݒ肵A{ Publisher ̋쓮~B
     * AőPRV[}̑oĂяoꍇB
     *
     * @else
     * @brief Task terminate function
     *
     * ACE_Task::release() override function.
     * Set driven flag to false, and terminate the operation of this Publisher.
     * However, the consumer's sending process may be invoked once or less.
     *
     *
     * @endif
     */
    virtual void release();
    
  protected:
    
  private:
    InPortConsumer* m_consumer;
    bool m_running;
    unsigned int m_usec;
  };
};     // namespace RTC
#endif // PublisherPeriodic_h

